/*
   This is a part of the LSCP2 Settings IPC Module Source code.

   Copyright(c) 2004 Sergey Gagarin a.k.a. Seg@
   mailto:inform-sega@freemail.ru
   http://www.litestep.bip.ru/

   Copyright(c) 2003 Niversoft 
   http//niversoft.dyndns.org
   info@niversoft.dyndns.org

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-------------------------------------------------------------------------------------

#include "settingsipc_svr.h"
#include "settingsipcdefs.h"

settingsipc_svr *module = NULL; // The module

#include <stdio.h> 
#include <stdlib.h> 
#include <string.h> 
#include <windows.h> 
#include <shellapi.h>
#include <vector>

using namespace std;

BOOL __stdcall BangCallback(LPCSTR pszName, LPARAM lParam);
BOOL __stdcall ModulesCallback(LPCSTR pszPath, DWORD dwFlags, LPARAM lParam);
BOOL __stdcall RevIDCallback(LPCSTR pszRevID, LPARAM lParam);
HRESULT EnumHotkeys(vector<EnumLSDataInfo> *v);

BOOL SendDataToWindow(vector<EnumLSDataInfo> &v, unsigned long id, HWND hWndReceiver);

HINSTANCE hInst;

void bangLSCP(HWND caller, LPCTSTR szArgs)
{
	char szPath[MAX_PATH], szLSCP[MAX_PATH];
	GetModuleFileName(hInst, szPath, MAX_LINE_LENGTH);
	strrchr(szPath,'\\')[1] = 0;
	strcpy(szLSCP,szPath);
	strcat(szLSCP,"lscp2.exe");
	HINSTANCE hinst = ShellExecute(NULL, NULL, szLSCP, szArgs, szPath, SW_SHOWNORMAL);
	if (hinst <= (HINSTANCE)32)
	{
		char szErrorMsg[1024];
		strcpy(szErrorMsg,"Cannot run LSCP2\nExpected location is \'");
		strcat(szErrorMsg,szLSCP);
		strcat(szErrorMsg,"\'");
		MessageBox(NULL,szErrorMsg,"Fatal error",MB_OK);
	}
}
 
int initModuleEx(HWND parentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
	int code;
	AddBangCommand("!lscp",bangLSCP);
	hInst = dllInst;
	if (!module) module = new settingsipc_svr(dllInst, code);
	return code;
}

void quitModule(HINSTANCE dllInst)
{
	RemoveBangCommand("!lscp");
	delete module;
	module = NULL;
}

settingsipc_svr::settingsipc_svr(HINSTANCE dllInst, int& code)
{
	code = 1;
   
	if (!GetLitestepWnd()) return;
   
	DWORD ThreadID;

	hMap  = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0, sizeof(TSharedMemory), SHAREDMEMNAME);
	hExec = CreateEvent(NULL, FALSE, FALSE, EVENT_EXEC);
	hDone = CreateEvent(NULL, FALSE, FALSE, EVENT_DONE);
	pMsg  = (TSharedMemory *)MapViewOfFile(hMap, FILE_MAP_WRITE, 0, 0, sizeof(TSharedMemory));

	m_Thread = CreateThread(NULL, 0, MainThreadProc, NULL, 0, &ThreadID);
   
	code = 0;
}

settingsipc_svr::~settingsipc_svr()
{
   TerminateThread(m_Thread, 0);	
   
   CloseHandle(hExec);
   CloseHandle(hDone);         
   UnmapViewOfFile(pMsg);
   CloseHandle(hMap);   
}


DWORD WINAPI settingsipc_svr::MainThreadProc(LPVOID lpParameter) // static member
{
	return module ? module->MainThreadProc() : 0;
}

DWORD settingsipc_svr::MainThreadProc()
{
	vector<EnumLSDataInfo> v;
	while (1)
	{
		WaitForSingleObject(hExec, INFINITE);

		v.clear();
		switch(module->pMsg->Action)
		{
		case eaNoAction:
			break;
		case eaGetLitestepWnd:
			pMsg->hWnd = GetLitestepWnd();
			break;
		case eaEnumLSData:
			switch(pMsg->defEnumData.uInfo)
			{
			case ELD_REVIDS:
				pMsg->hResult = EnumLSData(pMsg->defEnumData.uInfo, (FARPROC)RevIDCallback,	 (LPARAM)(&v));
				SendDataToWindow(v, ELD_REVIDS, pMsg->defEnumData.hwndReceiver);
				break;
			case ELD_BANGS:
				pMsg->hResult = EnumLSData(pMsg->defEnumData.uInfo, (FARPROC)BangCallback,	 (LPARAM)(&v));
				SendDataToWindow(v, ELD_REVIDS, pMsg->defEnumData.hwndReceiver);
				break;
			case ELD_MODULES:
				pMsg->hResult = EnumLSData(pMsg->defEnumData.uInfo, (FARPROC)ModulesCallback,(LPARAM)(&v));
				SendDataToWindow(v, ELD_REVIDS, pMsg->defEnumData.hwndReceiver);
				break;
			case  ELD_HOTKEYS:
				pMsg->hResult = EnumHotkeys(&v);
				SendDataToWindow(v, ELD_HOTKEYS, pMsg->defEnumData.hwndReceiver);
				break;
			default:
				pMsg->hResult = E_INVALIDARG;
			}
			break;
		case eaGetRCInt:
			pMsg->defGetRC.nRetValue = GetRCInt(pMsg->defGetRC.szKeyName, pMsg->nDefaultValue); 
			break;
		case eaGetRCBool:
			pMsg->defGetRC.nRetValue = GetRCBool(pMsg->defGetRC.szKeyName, pMsg->nDefaultValue);
			break;
		case eaGetRCBoolDef:
			pMsg->defGetRC.nRetValue = GetRCBoolDef(pMsg->defGetRC.szKeyName, pMsg->nDefaultValue);
			break;
		case eaGetRCColor:
			pMsg->defGetRC.nRetValue = GetRCColor(pMsg->defGetRC.szKeyName, pMsg->nDefaultValue);
			break;
		case eaGetRCString:
			pMsg->defGetRC.nRetValue = GetRCString(pMsg->defGetRC.szKeyName, pMsg->defGetRC.szRetValue, pMsg->defGetRC.szDefaultValue, pMsg->nMaxLength);
			break;
		case eaGetRCLine:
			pMsg->defGetRC.nRetValue = GetRCLine(pMsg->defGetRC.szKeyName, pMsg->defGetRC.szRetValue, pMsg->nMaxLength, pMsg->defGetRC.szDefaultValue);
			break;
		}
		SetEvent(hDone);
	}
	return 0;
}


BOOL SendDataToWindow(vector<EnumLSDataInfo> &v, unsigned long id, HWND hWndReceiver)
{
	BOOL bResult = FALSE;

	int packet_size = sizeof(int) + v.size() * sizeof(EnumLSDataInfo);

	char *data = new char[packet_size];
	EnumLSDataInfo *info = (EnumLSDataInfo*)(data+sizeof(int));

	* ((int*)data) = v.size();

	for (size_t i=0; i<v.size(); i++)
	{
		info[i] = v[i];
	}

	COPYDATASTRUCT cds;

	cds.cbData = packet_size;
	cds.dwData = id;
	cds.lpData = data;

	bResult = SendMessage(
                  hWndReceiver,
                  WM_COPYDATA,
                  (WPARAM) GetLitestepWnd(),
                  (LPARAM) &cds);
	return bResult;
}

void StoreInfo(vector<EnumLSDataInfo> *v, LPCSTR szString, DWORD dwFlags)
{
	EnumLSDataInfo info;
	strcpy(info.szString, szString);
	info.dwFlags = dwFlags;
	v->push_back(info);
}

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// RecIDCallback
//
BOOL __stdcall RevIDCallback(LPCSTR pszRevID, LPARAM lParam)
{
	StoreInfo((vector<EnumLSDataInfo> *)lParam, pszRevID, 0);
	return TRUE;
}

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ModulesCallback
//

BOOL __stdcall ModulesCallback(LPCSTR pszPath, DWORD dwFlags, LPARAM lParam)
{
	StoreInfo((vector<EnumLSDataInfo> *)lParam, pszPath, dwFlags);
	return TRUE;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// BangCallback
//
BOOL __stdcall BangCallback(LPCSTR pszName, LPARAM lParam)
{
	StoreInfo((vector<EnumLSDataInfo> *)lParam, pszName, 0);
	return TRUE;
}

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// EnumHotkeys - funciton enumerates defined hotkeys and send info
//               about'em to hwndReceiver
//
HRESULT EnumHotkeys(vector<EnumLSDataInfo> *v)
{
	FILE *f = LCOpen(NULL);
	if(f)
	{
		char buffer[MAX_LINE_LENGTH];

		char* tokens[4];
		char name[MAX_LINE_LENGTH];
		char fullpath[MAX_LINE_LENGTH];
		char full1[MAX_LINE_LENGTH];
		char full2[MAX_LINE_LENGTH];

		int i = 1;

		tokens[0] = name;
		tokens[1] = fullpath;
		tokens[2] = full1;
		tokens[3] =full2;
		name[0] = fullpath[0] = full1[0] = full2[0] = 0;
		while(LCReadNextConfig(f, "*Hotkey", buffer, sizeof(buffer)))
		{
			LCTokenize(buffer, tokens, 4, NULL);

			char* tmp;
			char* shortname;
	
			strcpy(buffer,fullpath);
			tmp = strtok(buffer, "\\");
			do {
				shortname = tmp;
				tmp = strtok(NULL, "\\");
			} while (tmp);

			strcat(fullpath, "|");
			strcat(fullpath, full1);
			strcat(fullpath, "|");
			strcat(fullpath, full2);
			StoreInfo(v, fullpath, 0);
		}		
	} 
	return S_OK;
}

